@@ -35,9 +35,9 @@
 #endif
 
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-idle.h>
 
 #include <telepathy-glib/account.h>
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/connection-manager.h>
 #include <telepathy-glib/util.h>
 #include <dbus/dbus-protocol.h>
@@ -63,6 +63,7 @@ typedef struct {
   GtkWidget *button_forget;
   GtkWidget *spinbutton_port;
   GtkWidget *enabled_checkbox;
+  GtkWidget *radiobutton_reuse;
 
   gboolean simple;
 
@@ -71,11 +72,14 @@ typedef struct {
   /* An EmpathyAccountWidget can be used to either create an account or
    * modify it. When we are creating an account, this member is set to TRUE */
   gboolean creating_account;
-  /* If we are creating a new account, this member is set to TRUE once the
-   * account has been created */
-  gboolean account_created;
 
-  EmpathyIdle *idle;
+  /* if TRUE, the GTK+ destroy signal has been fired and so the widgets
+   * embedded in this account widget can't be used any more
+   * workaround because some async callbacks can be called after the
+   * widget has been destroyed */
+  gboolean destroyed;
+
+  TpAccountManager *account_manager;
 
   gboolean dispose_run;
 } EmpathyAccountWidgetPriv;
@@ -110,8 +114,6 @@ account_widget_set_control_buttons_sensitivity (EmpathyAccountWidget *self,
       gtk_widget_set_sensitive (priv->apply_button, sensitive);
       gtk_widget_set_sensitive (
           priv->cancel_button, sensitive || priv->creating_account);
-
-      priv->contains_pending_changes = sensitive;
     }
 }
 
@@ -166,8 +168,12 @@ static void
 account_widget_entry_changed_cb (GtkEditable *entry,
     EmpathyAccountWidget *self)
 {
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
   account_widget_entry_changed_common (self, GTK_ENTRY (entry), FALSE);
   account_widget_handle_control_buttons_sensitivity (self);
+
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -209,6 +215,7 @@ account_widget_int_changed_cb (GtkWidget *widget,
     }
 
   account_widget_handle_control_buttons_sensitivity (self);
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -241,6 +248,7 @@ account_widget_checkbutton_toggled_cb (GtkWidget *widget,
     }
 
   account_widget_handle_control_buttons_sensitivity (self);
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -258,6 +266,7 @@ account_widget_forget_clicked_cb (GtkWidget *button,
   gtk_entry_set_text (GTK_ENTRY (priv->entry_password), "");
 
   account_widget_handle_control_buttons_sensitivity (self);
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -269,6 +278,8 @@ account_widget_password_changed_cb (GtkWidget *entry,
 
   str = gtk_entry_get_text (GTK_ENTRY (entry));
   gtk_widget_set_sensitive (priv->button_forget, !EMP_STR_EMPTY (str));
+
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -294,10 +305,52 @@ account_widget_jabber_ssl_toggled_cb (GtkWidget *checkbutton_ssl,
     }
 
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (priv->spinbutton_port), port);
+
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
-account_widget_setup_widget (EmpathyAccountWidget *self,
+account_widget_combobox_changed_cb (GtkWidget *widget,
+    EmpathyAccountWidget *self)
+{
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+  const gchar *value;
+  const GValue *v;
+  const gchar *default_value = NULL;
+  const gchar *param_name;
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter))
+    return;
+
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+  /* the param value is stored in the first column */
+  gtk_tree_model_get (model, &iter, 0, &value, -1);
+
+  param_name = g_object_get_data (G_OBJECT (widget), "param_name");
+
+  v = empathy_account_settings_get_default (priv->settings, param_name);
+  if (v != NULL)
+    default_value = g_value_get_string (v);
+
+  if (!tp_strdiff (value, default_value))
+    {
+      DEBUG ("Unset %s and restore to %s", param_name, default_value);
+      empathy_account_settings_unset (priv->settings, param_name);
+    }
+  else
+    {
+      DEBUG ("Setting %s to %s", param_name, value);
+      empathy_account_settings_set_string (priv->settings, param_name, value);
+    }
+
+  account_widget_handle_control_buttons_sensitivity (self);
+  priv->contains_pending_changes = TRUE;
+}
+
+void
+empathy_account_widget_setup_widget (EmpathyAccountWidget *self,
     GtkWidget *widget,
     const gchar *param_name)
 {
@@ -372,6 +425,41 @@ account_widget_setup_widget (EmpathyAccountWidget *self,
           G_CALLBACK (account_widget_checkbutton_toggled_cb),
           self);
     }
+  else if (GTK_IS_COMBO_BOX (widget))
+    {
+      /* The combo box's model has to contain the param value in its first
+       * column (as a string) */
+      const gchar *str;
+      GtkTreeModel *model;
+      GtkTreeIter iter;
+      gboolean valid;
+
+      str = empathy_account_settings_get_string (priv->settings, param_name);
+      model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+
+      valid = gtk_tree_model_get_iter_first (model, &iter);
+      while (valid)
+        {
+          gchar *name;
+
+          gtk_tree_model_get (model, &iter, 0, &name, -1);
+          if (!tp_strdiff (name, str))
+            {
+              gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget), &iter);
+              valid = FALSE;
+            }
+          else
+            {
+              valid = gtk_tree_model_iter_next (model, &iter);
+            }
+
+          g_free (name);
+        }
+
+      g_signal_connect (widget, "changed",
+          G_CALLBACK (account_widget_combobox_changed_cb),
+          self);
+    }
   else
     {
       DEBUG ("Unknown type of widget for param %s", param_name);
@@ -532,7 +620,7 @@ accounts_widget_generic_setup (EmpathyAccountWidget *self,
         }
 
       if (widget)
-        account_widget_setup_widget (self, widget, param->name);
+        empathy_account_widget_setup_widget (self, widget, param->name);
 
       g_free (param_name_formatted);
     }
@@ -559,7 +647,8 @@ account_widget_handle_params_valist (EmpathyAccountWidget *self,
           continue;
         }
 
-      account_widget_setup_widget (self, GTK_WIDGET (object), param_name);
+      empathy_account_widget_setup_widget (self, GTK_WIDGET (object),
+          param_name);
     }
 }
 
@@ -579,19 +668,41 @@ account_widget_account_enabled_cb (GObject *source_object,
   TpAccount *account = TP_ACCOUNT (source_object);
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
+  TpConnectionPresenceType presence;
+  gchar *message = NULL;
+  gchar *status = NULL;
 
   tp_account_set_enabled_finish (account, res, &error);
 
   if (error != NULL)
     {
-      DEBUG ("Could not automatically enable new account: %s", error->message);
+      DEBUG ("Could not enable the account: %s", error->message);
       g_error_free (error);
     }
   else
     {
-      priv->account_created = TRUE;
-      g_signal_emit (widget, signals[ACCOUNT_CREATED], 0);
+      /* only force presence if presence was offline, unknown or unset */
+      presence = tp_account_get_requested_presence (account, NULL, NULL);
+      switch (presence)
+        {
+        case TP_CONNECTION_PRESENCE_TYPE_OFFLINE:
+        case TP_CONNECTION_PRESENCE_TYPE_UNKNOWN:
+        case TP_CONNECTION_PRESENCE_TYPE_UNSET:
+          presence = tp_account_manager_get_most_available_presence (
+              priv->account_manager, &status, &message);
+          tp_account_request_presence_async (account, presence,
+              status, NULL, NULL, NULL);
+          break;
+        default:
+          /* do nothing if the presence is not offline */
+          break;
+        }
     }
+
+  /* unref widget - part of the workaround */
+  g_object_unref (widget);
+  g_free (message);
+  g_free (status);
 }
 
 static void
@@ -621,20 +732,25 @@ account_widget_applied_cb (GObject *source_object,
       if (priv->creating_account)
         {
           /* By default, when an account is created, we enable it. */
+
+          /* workaround to keep widget alive during async call */
+          g_object_ref (widget);
+
           tp_account_set_enabled_async (account, TRUE,
               account_widget_account_enabled_cb, widget);
+          g_signal_emit (widget, signals[ACCOUNT_CREATED], 0, account);
         }
       else if (priv->enabled_checkbox != NULL)
         {
           gboolean enabled_checked;
 
           enabled_checked =
-#ifndef HAVE_MOBLIN
-            gtk_toggle_button_get_active (
-                GTK_TOGGLE_BUTTON (priv->enabled_checkbox));
-#else
+#ifdef HAVE_MOBLIN
             nbtk_gtk_light_switch_get_active (
                 NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox));
+#else
+            gtk_toggle_button_get_active (
+                GTK_TOGGLE_BUTTON (priv->enabled_checkbox));
 #endif
 
           if (tp_account_is_enabled (account) && enabled_checked)
@@ -647,7 +763,13 @@ account_widget_applied_cb (GObject *source_object,
         }
     }
 
-  account_widget_set_control_buttons_sensitivity (widget, FALSE);
+  if (!priv->destroyed)
+    account_widget_set_control_buttons_sensitivity (widget, FALSE);
+
+  priv->contains_pending_changes = FALSE;
+
+  /* unref the widget - part of the workaround */
+  g_object_unref (widget);
 }
 
 static void
@@ -656,6 +778,17 @@ account_widget_apply_clicked_cb (GtkWidget *button,
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
 
+  if (priv->radiobutton_reuse != NULL)
+    {
+      gboolean reuse = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (
+            priv->radiobutton_reuse));
+
+      DEBUG ("Set register param: %d", !reuse);
+      empathy_account_settings_set_boolean (priv->settings, "register", !reuse);
+    }
+
+  /* workaround to keep widget alive during async call */
+  g_object_ref (self);
   empathy_account_settings_apply_async (priv->settings,
       account_widget_applied_cb, self);
 }
@@ -793,6 +926,15 @@ account_widget_build_msn (EmpathyAccountWidget *self,
     }
 }
 
+static gboolean
+account_widget_is_gtalk (EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  return !tp_strdiff (empathy_account_settings_get_icon_name (priv->settings),
+      "im-google-talk");
+}
+
 static void
 account_widget_build_jabber (EmpathyAccountWidget *self,
     const char *filename)
@@ -805,9 +947,7 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
   GtkWidget *label_example_gtalk, *label_example_jabber;
   gboolean is_gtalk;
 
-  is_gtalk = !tp_strdiff (
-      empathy_account_settings_get_icon_name (priv->settings),
-      "im-google-talk");
+  is_gtalk = account_widget_is_gtalk (self);
 
   if (priv->simple && !is_gtalk)
     {
@@ -1051,6 +1191,10 @@ static void
 account_widget_destroy_cb (GtkWidget *widget,
     EmpathyAccountWidget *self)
 {
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  /* set the destroyed flag - workaround */
+  priv->destroyed = TRUE;
+
   g_object_unref (self);
 }
 
@@ -1065,25 +1209,25 @@ empathy_account_widget_enabled_cb (TpAccount *account,
 
   if (priv->enabled_checkbox != NULL)
     {
-#ifndef HAVE_MOBLIN
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->enabled_checkbox),
-          enabled);
-#else
+#ifdef HAVE_MOBLIN
       nbtk_gtk_light_switch_set_active (
           NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox),
           enabled);
+#else
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->enabled_checkbox),
+          enabled);
 #endif /* HAVE_MOBLIN */
     }
 }
 
 static void
-#ifndef HAVE_MOBLIN
-account_widget_enabled_released_cb (GtkToggleButton *toggle_button,
-    gpointer user_data)
-#else
+#ifdef HAVE_MOBLIN
 account_widget_switch_flipped_cb (NbtkGtkLightSwitch *sw,
     gboolean state,
     gpointer user_data)
+#else
+account_widget_enabled_released_cb (GtkToggleButton *toggle_button,
+    gpointer user_data)
 #endif /* HAVE_MOBLIN */
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (user_data);
@@ -1097,7 +1241,10 @@ account_widget_switch_flipped_cb (NbtkGtkLightSwitch *sw,
   account = empathy_account_settings_get_account (priv->settings);
 
   /* Enable the account according to the value of the "Enabled" checkbox */
-  tp_account_set_enabled_async (account, state, NULL, NULL);
+  /* workaround to keep widget alive during async call */
+  g_object_ref (user_data);
+  tp_account_set_enabled_async (account, state,
+      account_widget_account_enabled_cb, user_data);
 }
 
 static void
@@ -1153,29 +1300,177 @@ do_get_property (GObject *object,
 }
 
 static void
-idle_state_change_cb (EmpathyIdle *idle,
-    GParamSpec *spec,
+presence_changed_cb (TpAccountManager *manager,
+    TpConnectionPresenceType state,
+    const gchar *status,
+    const gchar *message,
     EmpathyAccountWidget *self)
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-  TpConnectionPresenceType state;
 
-  state = empathy_idle_get_state (priv->idle);
+  if (priv->destroyed)
+    return;
 
   if (state > TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
     {
-      g_object_set (priv->apply_button, "label", GTK_STOCK_CONNECT, NULL);
+      /* We are online, display a Login button */
+      GtkWidget *image;
+
+      gtk_button_set_use_stock (GTK_BUTTON (priv->apply_button), FALSE);
+      gtk_button_set_label (GTK_BUTTON (priv->apply_button), _("L_og in"));
+
+      image = gtk_image_new_from_stock (GTK_STOCK_CONNECT,
+          GTK_ICON_SIZE_BUTTON);
+      gtk_button_set_image (GTK_BUTTON (priv->apply_button), image);
     }
   else
     {
-      g_object_set (priv->apply_button, "label", GTK_STOCK_APPLY, NULL);
+      /* We are offline, display a Save button */
+      gtk_button_set_image (GTK_BUTTON (priv->apply_button), NULL);
+      gtk_button_set_use_stock (GTK_BUTTON (priv->apply_button), TRUE);
+      gtk_button_set_label (GTK_BUTTON (priv->apply_button), GTK_STOCK_SAVE);
     }
 }
 
+static void
+account_manager_ready_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  EmpathyAccountWidget *self = EMPATHY_ACCOUNT_WIDGET (user_data);
+  TpAccountManager *account_manager = TP_ACCOUNT_MANAGER (source_object);
+  GError *error = NULL;
+  TpConnectionPresenceType state;
+
+  if (!tp_account_manager_prepare_finish (account_manager, result, &error))
+    {
+      DEBUG ("Failed to prepare account manager: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  state = tp_account_manager_get_most_available_presence (account_manager, NULL,
+      NULL);
+
+  /* simulate a presence change so the apply button will be changed
+   * if needed */
+  presence_changed_cb (account_manager, state, NULL, NULL, self);
+
+out:
+  g_object_unref (self);
+}
+
 #define WIDGET(cm, proto) \
   { #cm, #proto, "empathy-account-widget-"#proto".ui", \
     account_widget_build_##proto }
 
+static void
+add_enable_checkbox (EmpathyAccountWidget *self,
+    TpAccount *account)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+#ifdef HAVE_MOBLIN
+  GtkWidget *w;
+#else
+  GtkWidget *vbox = self->ui_details->widget;
+#endif
+  guint nb_rows, nb_columns;
+  gboolean is_enabled;
+
+  /* handle the "Enabled" checkbox. We only add it when modifying an account */
+  if (priv->creating_account || priv->table_common_settings == NULL)
+    return;
+
+  is_enabled = tp_account_is_enabled (account);
+
+#ifdef HAVE_MOBLIN
+  w = gtk_label_new (_("Account:"));
+  gtk_misc_set_alignment (GTK_MISC (w), 0, 0.5);
+
+  priv->enabled_checkbox = nbtk_gtk_light_switch_new ();
+
+  nbtk_gtk_light_switch_set_active (
+      NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox), is_enabled);
+
+  gtk_widget_show (w);
+#else
+  priv->enabled_checkbox =
+      gtk_check_button_new_with_label (_("Enabled"));
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->enabled_checkbox),
+      is_enabled);
+#endif /* HAVE_MOBLIN */
+
+  g_object_get (priv->table_common_settings, "n-rows", &nb_rows,
+      "n-columns", &nb_columns, NULL);
+
+  gtk_table_resize (GTK_TABLE (priv->table_common_settings), ++nb_rows,
+      nb_columns);
+
+#ifdef HAVE_MOBLIN
+  gtk_table_attach (GTK_TABLE (priv->table_common_settings),
+      w,
+      0, 1, nb_rows - 1, nb_rows,
+      GTK_FILL, 0, 0, 0);
+  gtk_table_attach (GTK_TABLE (priv->table_common_settings),
+      priv->enabled_checkbox,
+      1, nb_columns, nb_rows - 1, nb_rows,
+      GTK_EXPAND | GTK_FILL, 0, 0, 0);
+#else
+  gtk_box_pack_start (GTK_BOX (vbox), priv->enabled_checkbox, FALSE, FALSE, 0);
+  gtk_box_reorder_child (GTK_BOX (vbox), priv->enabled_checkbox, 0);
+#endif /* HAVE_MOBLIN */
+
+  gtk_widget_show (priv->enabled_checkbox);
+
+#ifdef HAVE_MOBLIN
+  g_signal_connect (G_OBJECT (priv->enabled_checkbox), "switch-flipped",
+      G_CALLBACK (account_widget_switch_flipped_cb), self);
+#else
+  g_signal_connect (G_OBJECT (priv->enabled_checkbox), "released",
+      G_CALLBACK (account_widget_enabled_released_cb), self);
+#endif /* HAVE_MOBLIN */
+}
+
+#ifndef HAVE_MOBLIN
+/* Moblin doesn't support registration */
+static void
+add_register_buttons (EmpathyAccountWidget *self,
+    TpAccount *account)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  const TpConnectionManagerProtocol *protocol;
+  GtkWidget *radiobutton_register;
+  GtkWidget *vbox = self->ui_details->widget;
+
+  if (!priv->creating_account)
+    return;
+
+  protocol = empathy_account_settings_get_tp_protocol (priv->settings);
+  if (protocol == NULL)
+    return;
+
+  if (!tp_connection_manager_protocol_can_register (protocol))
+    return;
+
+  if (account_widget_is_gtalk (self))
+    return;
+
+  priv->radiobutton_reuse = gtk_radio_button_new_with_label (NULL,
+      _("This account already exists on the server"));
+  radiobutton_register = gtk_radio_button_new_with_label (
+      gtk_radio_button_get_group (GTK_RADIO_BUTTON (priv->radiobutton_reuse)),
+      _("Create a new account on the server"));
+
+  gtk_box_pack_start (GTK_BOX (vbox), priv->radiobutton_reuse, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), radiobutton_register, FALSE, FALSE, 0);
+  gtk_box_reorder_child (GTK_BOX (vbox), priv->radiobutton_reuse, 0);
+  gtk_box_reorder_child (GTK_BOX (vbox), radiobutton_register, 1);
+  gtk_widget_show (priv->radiobutton_reuse);
+  gtk_widget_show (radiobutton_register);
+}
+#endif
+
 static void
 do_constructed (GObject *obj)
 {
@@ -1264,6 +1559,13 @@ do_constructed (GObject *obj)
           self);
     }
 
+  /* dup and init the account-manager */
+  priv->account_manager = tp_account_manager_dup ();
+
+  g_object_ref (self);
+  tp_account_manager_prepare_async (priv->account_manager, NULL,
+      account_manager_ready_cb, self);
+
   /* handle apply and cancel button */
   if (!priv->simple)
     {
@@ -1273,29 +1575,13 @@ do_constructed (GObject *obj)
 
       if (priv->creating_account)
         {
-          TpConnectionPresenceType state;
-          priv->idle = empathy_idle_dup_singleton ();
+          /* Assumre we are offline, display a Save button. We'll update
+           * it once the account manager is ready if needed */
+          priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
 
-          empathy_signal_connect_weak (priv->idle, "notify::state",
-              G_CALLBACK (idle_state_change_cb), obj);
-
-          state = empathy_idle_get_state (priv->idle);
-
-          if (state > TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
-            {
-              /* We are online, display a Login button */
-              GtkWidget *image;
-
-              priv->apply_button = gtk_button_new_with_mnemonic (_("L_og in"));
-              image = gtk_image_new_from_stock (GTK_STOCK_CONNECT,
-                  GTK_ICON_SIZE_BUTTON);
-              gtk_button_set_image (GTK_BUTTON (priv->apply_button), image);
-            }
-          else
-            {
-              /* We are offline, display a Save button */
-              priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
-            }
+          empathy_signal_connect_weak (priv->account_manager,
+              "most-available-presence-changed",
+              G_CALLBACK (presence_changed_cb), obj);
         }
       else
         {
@@ -1337,68 +1623,10 @@ do_constructed (GObject *obj)
           G_CALLBACK (empathy_account_widget_enabled_cb), self);
     }
 
-  /* handle the "Enabled" checkbox. We only add it when modifying an account */
-  if (!priv->creating_account && priv->table_common_settings != NULL)
-    {
-#ifdef HAVE_MOBLIN
-      GtkWidget *w;
+#ifndef HAVE_MOBLIN
+  add_register_buttons (self, account);
 #endif
-      guint nb_rows, nb_columns;
-      gboolean is_enabled;
-
-      is_enabled = tp_account_is_enabled (account);
-
-#ifndef HAVE_MOBLIN
-      priv->enabled_checkbox =
-          gtk_check_button_new_with_label (_("Enabled"));
-
-      gtk_toggle_button_set_active (
-          GTK_TOGGLE_BUTTON (priv->enabled_checkbox), is_enabled);
-#else
-      /* Translators: this is used only when built on a moblin platform */
-      w = gtk_label_new (_("Account:"));
-      gtk_misc_set_alignment (GTK_MISC (w), 0, 0.5);
-
-      priv->enabled_checkbox = nbtk_gtk_light_switch_new ();
-
-      nbtk_gtk_light_switch_set_active (
-          NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox), is_enabled);
-
-      gtk_widget_show (w);
-#endif /* HAVE_MOBLIN */
-
-      g_object_get (priv->table_common_settings, "n-rows", &nb_rows,
-          "n-columns", &nb_columns, NULL);
-
-      gtk_table_resize (GTK_TABLE (priv->table_common_settings), ++nb_rows,
-          nb_columns);
-
-#ifndef HAVE_MOBLIN
-      gtk_table_attach (GTK_TABLE (priv->table_common_settings),
-          priv->enabled_checkbox,
-          0, nb_columns, nb_rows - 1, nb_rows,
-          GTK_EXPAND | GTK_FILL, 0, 0, 0);
-#else
-      gtk_table_attach (GTK_TABLE (priv->table_common_settings),
-          w,
-          0, 1, nb_rows - 1, nb_rows,
-          GTK_FILL, 0, 0, 0);
-      gtk_table_attach (GTK_TABLE (priv->table_common_settings),
-          priv->enabled_checkbox,
-          1, nb_columns, nb_rows - 1, nb_rows,
-          GTK_EXPAND | GTK_FILL, 0, 0, 0);
-#endif /* HAVE_MOBLIN */
-
-      gtk_widget_show (priv->enabled_checkbox);
-
-#ifndef HAVE_MOBLIN
-      g_signal_connect (G_OBJECT (priv->enabled_checkbox), "released",
-          G_CALLBACK (account_widget_enabled_released_cb), self);
-#else
-      g_signal_connect (G_OBJECT (priv->enabled_checkbox), "switch-flipped",
-          G_CALLBACK (account_widget_switch_flipped_cb), self);
-#endif /* HAVE_MOBLIN */
-    }
+  add_enable_checkbox (self, account);
 
   /* hook up to widget destruction to unref ourselves */
   g_signal_connect (self->ui_details->widget, "destroy",
@@ -1437,10 +1665,10 @@ do_dispose (GObject *obj)
       priv->settings = NULL;
     }
 
-  if (priv->idle != NULL)
+  if (priv->account_manager != NULL)
     {
-      g_object_unref (priv->idle);
-      priv->idle = NULL;
+      g_object_unref (priv->account_manager);
+      priv->account_manager = NULL;
     }
 
   if (G_OBJECT_CLASS (empathy_account_widget_parent_class)->dispose != NULL)
@@ -1507,9 +1735,9 @@ empathy_account_widget_class_init (EmpathyAccountWidgetClass *klass)
   signals[ACCOUNT_CREATED] =
       g_signal_new ("account-created", G_TYPE_FROM_CLASS (klass),
           G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-          g_cclosure_marshal_VOID__VOID,
+          g_cclosure_marshal_VOID__POINTER,
           G_TYPE_NONE,
-          0);
+          1, G_TYPE_OBJECT);
 
   signals[CANCELLED] =
       g_signal_new ("cancelled", G_TYPE_FROM_CLASS (klass),
@@ -1551,11 +1779,6 @@ empathy_account_widget_contains_pending_changes (EmpathyAccountWidget *widget)
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
 
-  if (priv->creating_account && !priv->account_created)
-    /* We always want to warn the user if he's in the process of creating a
-     * new account which hasn't been actually created yet. */
-    return TRUE;
-
   return priv->contains_pending_changes;
 }
 
