@@ -42,6 +42,7 @@
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-images.h>
 #include <libempathy-gtk/empathy-new-message-dialog.h>
+#include <libempathy-gtk/empathy-new-call-dialog.h>
 #include <libempathy-gtk/empathy-notify-manager.h>
 
 #include "empathy-accounts-dialog.h"
@@ -208,9 +209,10 @@ static void
 status_icon_update_tooltip (EmpathyStatusIcon *icon)
 {
 	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
-	gchar                 *tooltip = NULL;
 
 	if (priv->event) {
+		gchar *tooltip = NULL;
+
 		if (priv->event->message != NULL)
 				tooltip = g_markup_printf_escaped ("<i>%s</i>\n%s",
 								   priv->event->header,
@@ -219,13 +221,24 @@ status_icon_update_tooltip (EmpathyStatusIcon *icon)
 				tooltip = g_markup_printf_escaped ("<i>%s</i>",
 								   priv->event->header);
 		gtk_status_icon_set_tooltip_markup (priv->icon, tooltip);
+		g_free (tooltip);
 	} else {
-		tp_account_manager_get_most_available_presence (
-			priv->account_manager, &tooltip, NULL);
-		gtk_status_icon_set_tooltip_text (priv->icon, tooltip);
-	}
+		TpConnectionPresenceType type;
+		gchar *msg;
 
-	g_free (tooltip);
+		type = tp_account_manager_get_most_available_presence (
+			priv->account_manager, NULL, &msg);
+
+		if (!EMP_STR_EMPTY (msg)) {
+			gtk_status_icon_set_tooltip_text (priv->icon, msg);
+		}
+		else {
+			gtk_status_icon_set_tooltip_text (priv->icon,
+						empathy_presence_get_default_message (type));
+		}
+
+		g_free (msg);
+	}
 }
 
 static void
@@ -451,6 +464,13 @@ status_icon_new_message_cb (GtkAction         *action,
 	empathy_new_message_dialog_show (NULL);
 }
 
+static void
+status_icon_new_call_cb (GtkAction         *action,
+			    EmpathyStatusIcon *icon)
+{
+	empathy_new_call_dialog_show (NULL);
+}
+
 static void
 status_icon_quit_cb (GtkAction         *action,
 		     EmpathyStatusIcon *icon)
@@ -512,6 +532,7 @@ status_icon_create_menu (EmpathyStatusIcon *icon)
 	empathy_builder_connect (gui, icon,
 			      "show_list", "toggled", status_icon_show_hide_window_cb,
 			      "new_message", "activate", status_icon_new_message_cb,
+			      "new_call", "activate", status_icon_new_call_cb,
 			      "quit", "activate", status_icon_quit_cb,
 			      NULL);
 
